@@ -4,7 +4,9 @@ import importlib
 import importlib.util as util
 import inspect
 import json
+import math
 import os
+
 if os.name == 'nt':
     import winreg
 else:
@@ -85,7 +87,7 @@ class WorkflowEngine:
             if len(installdir) == 0:
                 return
             else:
-                self.set_db_path(installdir)        
+                self.set_db_path(installdir)
                 db_folder = installdir
         if pythonpath is None or len(pythonpath) == 0:
             if os.name == 'nt':
@@ -121,7 +123,7 @@ class WorkflowEngine:
         self.previous_step = None
         self.step_nr = 0
         self.step_input = None
-        self.runlog = ""
+        self.runlog = []
         self.variables = {}  # Dictionary to hold WorkflowEngine variables
 
     def get_input_parameter(self, as_dictionary: bool = False) -> any:
@@ -135,7 +137,7 @@ class WorkflowEngine:
                 if isinstance(self.input_parameter, str):
                     self.input_parameter = self.input_parameter.replace("true", "True").replace("false", "False")
                 self.input_parameter = eval(self.input_parameter)
-        self.print_log(f"Got input parameter {str(self.input_parameter)}")
+        self.print_log(f"Got input parameter {str(self.input_parameter)}", "Processing Input")
         return self.input_parameter
 
     def open(self, filepath: str, as_xml: bool = False) -> any:
@@ -155,8 +157,27 @@ class WorkflowEngine:
                 with open(filepath, "rb") as binary_file:
                     # Read the whole file at once
                     content = binary_file.read()
-                str_content = content[24:-1].decode("utf-8", errors='ignore')
+                str_content = content.decode("utf-8", errors='ignore')
+                str_content = "".join([x for x in str_content if x != '' and x != '']).strip().strip('\x00')
+                # remain =math.ceil((len(str_content)/4) - int(len(str_content)/4))
+                # str_content = str_content[0:len(str_content)-remain]
+                idx = 0
+                if str_content.__contains__("fV0"):
+                    str_content = str_content.split("fV0")[0][1:] + "fV0==="
+                if str_content.__contains__("In1d"):
+                    if str_content.index("In1d") + 4 > idx:
+                        idx = str_content.index("In1d") + 4
+                if idx > 0:
+                    str_content = str_content[0:idx]
                 decoded = base64.b64decode(str_content).decode("utf-8", errors='ignore')
+                t = 1
+                ## fV0=}]  of In1d = "}]
+                eq = "="
+
+                # idx = decoded.index("/*-  Zk
+                #  }]")
+                # if idx > 0:
+                #     decoded = decoded[0:idx+2]
                 dict_list = json.loads(decoded)
                 return dict_list
             else:
@@ -281,13 +302,18 @@ class WorkflowEngine:
             # It is a Visio Object!
             visio = ordered_dict
             return visio.get_flow()
+        # route for .flw flows
         if isinstance(ordered_dict, list):
             retn = []
             for rw in ordered_dict:
                 tmp = self.dynamic_object()
                 for k, v in rw.items():
-                    setattr(tmp, k, v)
+                    ky = k
+                    if ky.lower() == "class":
+                        ky = "classname"
+                    setattr(tmp, ky, v)
                 retn.append(tmp)
+            # return .flw flow steps
             return retn
         connectors = []
         shapes = []
@@ -723,7 +749,7 @@ class WorkflowEngine:
                                 if not str(step.module).__contains__("\\") and str(step.module).lower().__contains__(".py"):
                                     step.module = f"{self.packages_folder}\\BPMN_RPA\\Scripts\\{step.module}"
                                 if not str(step.module).__contains__(":") and str(step.module).__contains__("\\") and str(
-                                    step.module).__contains__(".py"):
+                                        step.module).__contains__(".py"):
                                     step.module = f"{self.packages_folder}\\{step.module}"
                             if str(step.module).lower().__contains__(".py"):
                                 spec = util.spec_from_file_location(step.module, step.module)
@@ -900,10 +926,10 @@ class WorkflowEngine:
             step_time = datetime.now().strftime("%H:%M:%S")
             if self.step_nr is not None:
                 print(f"{step_time}: Step {self.step_nr} - {result}")
-                self.runlog += f"{step_time}: Step {self.step_nr} - {result}"
+                self.runlog.append(f"{step_time}: Step {self.step_nr} - {result}")
             else:
                 print(f"{step_time}: {result}")
-                self.runlog += f"{step_time}: {result}"
+                self.runlog.append(f"{step_time}: {result}")
                 self.step_nr = ""
             result = result.replace("'", "''").strip()
             if len(status) > 0:
@@ -1042,7 +1068,7 @@ class WorkflowEngine:
                         else:
                             name = name.__str__()
                     end_result = f"loopitem '{name}' returned."
-                    self.print_log(end_result)
+                    self.print_log(end_result, "Looping")
                     return loopvar.items[loopvar.counter]
                 else:
                     return output_previous_step
@@ -1252,15 +1278,22 @@ class SQL:
         if not hasattr(self, "connection"):
             return
         self.connection.execute(sql)
-        self.connection.commit()
-        if len(tablename) > 0:
-            try:
-                return self.get_inserted_id(tablename)
-            except Exception as ex:
-                self.set_error(ex)
-                raise Exception(self.error)
+        if not sql.lower().startswith("select"):
+            self.connection.commit()
+            if len(tablename) > 0:
+                try:
+                    return self.get_inserted_id(tablename)
+                except Exception as ex:
+                    self.set_error(ex)
+                    raise Exception(self.error)
+            else:
+                return None
         else:
-            return None
+            row = self.connection.cursor().fetchone()
+            if row is not None:
+                return row[0]
+            else:
+                return None
 
     def set_error(self, ex: any):
         """
@@ -1639,8 +1672,9 @@ class Visio:
             setattr(retn, k, v)
         return retn
 
+
 # Test
 # engine = WorkflowEngine()
-# doc = engine.open(fr"c:\\temp\\test2.flw")  # c:\\temp\\test.xml
+# doc = engine.open(fr"C:\temp\t.flw")  # c:\\temp\\test.xml
 # steps = engine.get_flow(doc)
 # engine.run_flow(steps)
